@@ -29,10 +29,14 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <stdlib.h>
-#include <execinfo.h>
 #include <wordexp.h>
 #include <fnmatch.h>
 #include <time.h>
+#include <limits.h>
+
+#ifdef HAVE_EXECINFO_H
+#include <execinfo.h>
+#endif
 
 #include "fsarchiver.h"
 #include "syncthread.h"
@@ -354,6 +358,7 @@ int exec_command(char *command, int cmdbufsize, int *exitst, char *stdoutbuf, in
         close(pfildes1[1]); // close excess fildes
         dup2(pfildes2[1],2); // make 1 same as write-to end of pipe
         close(pfildes2[1]); // close excess fildes
+        setenv("LC_ALL", "C", 1); // kill internationalization
         execvp(pathtoprog, argv);
         errprintf("execvp(%s) failed\n", pathtoprog); // still around? exec failed
         wordfree(&p);
@@ -392,9 +397,9 @@ int exec_command(char *command, int cmdbufsize, int *exitst, char *stdoutbuf, in
         
         // read the remaining data in the pipes 
         if ((stdoutbuf!=NULL) && (outpos+1 < stdoutsize))
-            res=read(mystdout, stdoutbuf+outpos, stdoutsize-outpos-1);
+            read(mystdout, stdoutbuf+outpos, stdoutsize-outpos-1);
         if ((stderrbuf!=NULL) && (errpos+1 < stderrsize))
-            res=read(mystderr, stderrbuf+errpos, stderrsize-errpos-1);
+            read(mystderr, stderrbuf+errpos, stderrsize-errpos-1);
         
         msgprintf(MSG_VERB1, "command [%s] returned %d\n", command, WEXITSTATUS(status));
         if (exitst)
@@ -420,9 +425,9 @@ int generate_random_tmpdir(char *buffer, int bufsize, int n)
     abstime=tv.tv_sec;
         localtime_r(&abstime, &tbreak);
     
-    snprintf(buffer, bufsize, "/tmp/fsa/%.4d%.2d%.2d-%.2d%.2d%.2d-%.2d",
+    snprintf(buffer, bufsize, "/tmp/fsa/%.4d%.2d%.2d-%.2d%.2d%.2d-%.8x-%.2d",
         tbreak.tm_year+1900, tbreak.tm_mon+1, tbreak.tm_mday,
-         tbreak.tm_hour, tbreak.tm_min, tbreak.tm_sec, n);
+         tbreak.tm_hour, tbreak.tm_min, tbreak.tm_sec, (u32)tv.tv_usec, n);
     return 0;
 }
 
@@ -549,6 +554,7 @@ u64 stats_errcount(cstats stats)
 
 int format_stacktrace(char *buffer, int bufsize)
 {
+#ifdef HAVE_EXECINFO_H
     const int stack_depth=20;
     void *temp[stack_depth];
     char **strings;
@@ -565,6 +571,7 @@ int format_stacktrace(char *buffer, int bufsize)
             strlcatf(buffer, bufsize, "%s\n", strings[i]);
         free(strings);
     }
+#endif
     
     return 0;
 }
@@ -609,4 +616,3 @@ int get_path_to_volume(char *newvolbuf, int bufsize, char *basepath, long curvol
     
     return 0;
 }
-
