@@ -65,7 +65,7 @@ int xfs_check_compatibility(u64 compat, u64 ro_compat, u64 incompat, u64 log_inc
     }
 }
 
-int xfs_mkfs(cdico *d, char *partition, char *fsoptions)
+int xfs_mkfs(cdico *d, char *partition, char *fsoptions, char *mkfslabel, char *mkfsuuid)
 {
     char stdoutbuf[2048];
     char command[2048];
@@ -105,7 +105,9 @@ int xfs_mkfs(cdico *d, char *partition, char *fsoptions)
 
     strlcatf(mkfsopts, sizeof(mkfsopts), " %s ", fsoptions);
 
-    if (dico_get_string(d, 0, FSYSHEADKEY_FSLABEL, buffer, sizeof(buffer))==0 && strlen(buffer)>0)
+    if (strlen(mkfslabel) > 0)
+        strlcatf(mkfsopts, sizeof(mkfsopts), " -L '%.12s' ", mkfslabel);
+    else if (dico_get_string(d, 0, FSYSHEADKEY_FSLABEL, buffer, sizeof(buffer))==0 && strlen(buffer)>0)
         strlcatf(mkfsopts, sizeof(mkfsopts), " -L '%.12s' ", buffer);
 
     if ((dico_get_u64(d, 0, FSYSHEADKEY_FSXFSBLOCKSIZE, &temp64)==0) && (temp64%512==0) && (temp64>=512) && (temp64<=65536))
@@ -121,11 +123,10 @@ int xfs_mkfs(cdico *d, char *partition, char *fsoptions)
     if (xfs_check_compatibility(sb_features_compat, sb_features_ro_compat, sb_features_incompat, sb_features_log_incompat)!=0)
         return -1;
 
-    // Preserve version 4 of XFS if the original filesystem was an XFS v4 or if
-    // the original filesystem was saved with fsarchiver <= 0.6.19 which does
-    // not store the XFS version in the metadata: make an XFSv4 if unsure for
-    // better compatibility and as upgrading later is easier than downgrading
-    // if the user gets an XFSv4 and wanted an XFSv5
+    // Preserve version 4 of XFS if the original filesystem was an XFSv4 or if
+    // it was saved with fsarchiver <= 0.6.19 which does not store the XFS
+    // version in the metadata: make an XFSv4 if unsure for better compatibility,
+    // as upgrading later is easier than downgrading
     if ((dico_get_u64(d, 0, FSYSHEADKEY_FSXFSVERSION, &temp64)!=0) || (temp64==XFS_SB_VERSION_4))
         xfsver = XFS_SB_VERSION_4;
     else
@@ -134,8 +135,8 @@ int xfs_mkfs(cdico *d, char *partition, char *fsoptions)
     // Unfortunately it is impossible to preserve the UUID (stamped on every
     // metadata block) of an XFSv5 filesystem with mkfs.xfs < 4.3.0.
     // Restoring with a new random UUID would work but could prevent the system
-    // from booting if this is a boot/root filesystem and grub/fstab often use
-    // the UUID to identify it. Hence it is much safer to restore it as an
+    // from booting if this is a boot/root filesystem because grub/fstab often
+    // use the UUID to identify it. Hence it is much safer to restore it as an
     // XFSv4 and it also provides a better compatibility with older kernels.
     // Hence this is the safest option.
     // More details: https://github.com/fdupoux/fsarchiver/issues/4
@@ -162,9 +163,9 @@ int xfs_mkfs(cdico *d, char *partition, char *fsoptions)
     }
 
     // Determine if the "finobt" mkfs option should be enabled (free inode btree)
-    // - starting Linux 3.16 XFS has added a btree that tracks free inodes
+    // - starting with linux-3.16 XFS has added a btree that tracks free inodes
     // - this feature relies on the new v5 on-disk format but it is optional
-    // - this feature enabled by default when using xfsprogs 3.2.3 or later
+    // - this feature is enabled by default when using xfsprogs 3.2.3 or later
     // - this feature will be enabled if the original filesystem was XFSv5 and had it
     if (xfstoolsver >= PROGVER(3,2,1)) // only use "finobt" option when it is supported by mkfs
     {
@@ -172,12 +173,37 @@ int xfs_mkfs(cdico *d, char *partition, char *fsoptions)
         strlcatf(mkfsopts, sizeof(mkfsopts), " -m finobt=%d ", (int)optval);
     }
 
-    // ---- attempt to preserve UUID of the filesystem
+    // Determine if the "rmapbt" mkfs option should be enabled (reverse mapping btree)
+    // - starting with linux-4.8 XFS has added a btree that maps filesystem blocks
+    //   to their owner
+    // - this feature relies on the new v5 on-disk format but it is optional
+    // - this feature will be enabled if the original filesystem was XFSv5 and had it
+    if (xfstoolsver >= PROGVER(4,8,0)) // only use "rmapbt" option when it is supported by mkfs
+    {
+        optval = ((xfsver==XFS_SB_VERSION_5) && (sb_features_ro_compat & XFS_SB_FEAT_RO_COMPAT_RMAPBT));
+        strlcatf(mkfsopts, sizeof(mkfsopts), " -m rmapbt=%d ", (int)optval);
+    }
+
+    // Determine if the "reflink" mkfs option should be enabled
+    // - starting with linux-4.9 XFS has added support for reflinked files
+    // - this feature relies on the new v5 on-disk format but it is optional
+    // - this feature will be enabled if the original filesystem was XFSv5 and had it
+    if (xfstoolsver >= PROGVER(4,9,0)) // only use "reflink" option when it is supported by mkfs
+    {
+        optval = ((xfsver==XFS_SB_VERSION_5) && (sb_features_ro_compat & XFS_SB_FEAT_RO_COMPAT_REFLINK));
+        strlcatf(mkfsopts, sizeof(mkfsopts), " -m reflink=%d ", (int)optval);
+    }
+
+    // Attempt to preserve UUID of the filesystem
     // - the "-m uuid=<UUID>" option in mkfs.xfs was added in mkfs.xfs 4.3.0 and is the best way to set UUIDs
     // - the UUID of XFSv4 can be successfully set using either xfs_admin or mkfs.xfs >= 4.3.0
     // - it is impossible to set both types of UUIDs of an XFSv5 filesystem using xfsprogs < 4.3.0
     //   for this reason the XFS version is forced to v4 if xfsprogs version < 4.3.0
-    if (dico_get_string(d, 0, FSYSHEADKEY_FSUUID, uuid, sizeof(uuid))==0 && strlen(uuid)==36)
+    if (strlen(mkfsuuid) > 0)
+        snprintf(uuid, sizeof(uuid), "%s", mkfsuuid);
+    else if (dico_get_string(d, 0, FSYSHEADKEY_FSUUID, buffer, sizeof(buffer))==0)
+        snprintf(uuid, sizeof(uuid), "%s", buffer);
+    if (strlen(uuid)==36)
     {
         if (xfstoolsver >= PROGVER(4,3,0))
             strlcatf(mkfsopts, sizeof(mkfsopts), " -m uuid=%s ", uuid);
@@ -191,17 +217,30 @@ int xfs_mkfs(cdico *d, char *partition, char *fsoptions)
     // - when CRCs are enabled via -m crc=1, the ftype functionality is always enabled
     // - ftype is madatory in XFSv5 volumes but it is optional for XFSv4 volumes
     // - the "ftype" option must be specified after the "crc" option in mkfs.xfs < 4.2.0:
-    //   http://oss.sgi.com/cgi-bin/gitweb.cgi?p=xfs/cmds/xfsprogs.git;a=commit;h=b990de8ba4e2df2bc76a140799d3ddb4a0eac4ce
+    //   https://git.kernel.org/cgit/fs/xfs/xfsprogs-dev.git/commit/?id=b990de8ba4e2df2bc76a140799d3ddb4a0eac4ce
     // - do not set ftype=1 with crc=1 as mkfs.xfs may fail when both options are enabled (at least with xfsprogs-3.2.2)
     // - XFSv4 with ftype=1 is supported since linux-3.13. We purposely always
     //   disable ftype for V4 volumes to keep them compatible with older kernels
     if (xfstoolsver >= PROGVER(3,2,0)) // only use "ftype" option when it is supported by mkfs
     {
-        // crc is already set to 1 when it is XFSv5 hence do not set ftype=1 with XFSv5
+        // ftype is already set to 1 when it is XFSv5
         if (xfsver==XFS_SB_VERSION_4)
             strlcatf(mkfsopts, sizeof(mkfsopts), " -n ftype=0 ");
     }
 
+    // Determine if the "sparse" mkfs option should be enabled (sparse inode allocation)
+    // - starting with linux-4.2 XFS can allocate discontinuous inode chunks
+    // - this feature relies on the new v5 on-disk format but it is optional
+    // - this feature will be enabled if the original filesystem was XFSv5 and had it
+    // - this feature is supported since mkfs.xfs 4.2.0, but unfortunately
+    //   mkfs.xfs 4.5.0 in RHEL 7.3 carries a custom patch removing the option,
+    //   hence require mkfs.xfs 4.7.0, next version after 4.5.0
+    if (xfstoolsver >= PROGVER(4,7,0)) // only use "sparse" option when it is supported by mkfs
+    {
+        optval = ((xfsver==XFS_SB_VERSION_5) && (sb_features_incompat & XFS_SB_FEAT_INCOMPAT_SPINODES));
+        strlcatf(mkfsopts, sizeof(mkfsopts), " -i sparse=%d ", (int)optval);
+    }
+
     // ---- create the new filesystem using mkfs.xfs
     if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "mkfs.xfs -f %s %s", partition, mkfsopts)!=0 || exitst!=0)
     {   errprintf("command [%s] failed\n", command);
