@@ -1,7 +1,7 @@
 /*
  * fsarchiver: Filesystem Archiver
  *
- * Copyright (C) 2008-2010 Francois Dupoux.  All rights reserved.
+ * Copyright (C) 2008-2016 Francois Dupoux.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -22,6 +22,7 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include <byteswap.h>
+#include <linux/types.h>
 
 typedef uint8_t u8;
 typedef int8_t s8;
@@ -32,10 +33,6 @@ typedef int32_t s32;
 typedef uint64_t u64;
 typedef int64_t s64;
 
-typedef uint16_t __le16;
-typedef uint32_t __le32;
-typedef uint64_t __le64;
-
 #define le8_to_cpu(v) (v)
 #define cpu_to_le8(v) (v)
 
